@@ -1,5 +1,5 @@
-use std::convert::TryFrom;
 use std::borrow::Borrow;
+use std::convert::TryFrom;
 
 use crate::imp;
 
@@ -8,6 +8,10 @@ use crate::imp;
 pub struct HeaderName(imp::HeaderName);
 impl<'a> TryFrom<&'a str> for HeaderName {
     type Error = imp::Error;
+    #[cfg_attr(
+        not(any(feature = "use_hyper", feature = "use_async_h1")),
+        allow(unused_variables)
+    )]
     #[inline]
     fn try_from(t: &'a str) -> Result<HeaderName, Self::Error> {
         #[cfg(feature = "use_async_h1")]
@@ -15,11 +19,15 @@ impl<'a> TryFrom<&'a str> for HeaderName {
         #[cfg(feature = "use_hyper")]
         return Ok(HeaderName(imp::HeaderName::try_from(t)?));
         #[cfg(not(any(feature = "use_hyper", feature = "use_async_h1")))]
-        return Err(imp::Error{})
+        return Err(imp::Error {});
     }
 }
 impl<'a> TryFrom<&'a [u8]> for HeaderName {
     type Error = imp::Error;
+    #[cfg_attr(
+        not(any(feature = "use_hyper", feature = "use_async_h1")),
+        allow(unused_variables)
+    )]
     #[inline]
     fn try_from(t: &'a [u8]) -> Result<Self, Self::Error> {
         #[cfg(feature = "use_async_h1")]
@@ -27,11 +35,15 @@ impl<'a> TryFrom<&'a [u8]> for HeaderName {
         #[cfg(feature = "use_hyper")]
         return Ok(HeaderName(imp::HeaderName::from_bytes(t)?));
         #[cfg(not(any(feature = "use_hyper", feature = "use_async_h1")))]
-        return Err(imp::Error{})
+        return Err(imp::Error {});
     }
 }
 impl TryFrom<Vec<u8>> for HeaderName {
     type Error = imp::Error;
+    #[cfg_attr(
+        not(any(feature = "use_hyper", feature = "use_async_h1")),
+        allow(unused_variables)
+    )]
     #[inline]
     fn try_from(t: Vec<u8>) -> Result<Self, Self::Error> {
         #[cfg(feature = "use_async_h1")]
@@ -39,7 +51,7 @@ impl TryFrom<Vec<u8>> for HeaderName {
         #[cfg(feature = "use_hyper")]
         return Ok(HeaderName(imp::HeaderName::from_bytes(&t)?));
         #[cfg(not(any(feature = "use_hyper", feature = "use_async_h1")))]
-        return Err(imp::Error{})
+        return Err(imp::Error {});
     }
 }
 
@@ -53,7 +65,7 @@ impl<'a> From<&'a imp::HeaderName> for &'a HeaderName {
     #[inline]
     fn from(t: &'a imp::HeaderName) -> Self {
         //safe because repr(transparent)
-        unsafe{std::mem::transmute(t)}
+        unsafe { std::mem::transmute(t) }
     }
 }
 impl AsRef<str> for HeaderName {
@@ -92,6 +104,10 @@ impl Borrow<str> for HeaderName {
     }
 }
 impl PartialEq<str> for HeaderName {
+    #[cfg_attr(
+        not(any(feature = "use_hyper", feature = "use_async_h1")),
+        allow(unused_variables)
+    )]
     #[inline]
     fn eq(&self, other: &str) -> bool {
         #[cfg(feature = "use_hyper")]
@@ -115,6 +131,10 @@ impl<'a> TryFrom<&'a str> for HeaderValue {
 }
 impl<'a> TryFrom<&'a [u8]> for HeaderValue {
     type Error = imp::Error;
+    #[cfg_attr(
+        not(any(feature = "use_hyper", feature = "use_async_h1")),
+        allow(unused_variables)
+    )]
     #[inline]
     fn try_from(t: &'a [u8]) -> Result<Self, Self::Error> {
         #[cfg(feature = "use_async_h1")]
@@ -122,7 +142,7 @@ impl<'a> TryFrom<&'a [u8]> for HeaderValue {
         #[cfg(feature = "use_hyper")]
         return Ok(HeaderValue(imp::HeaderValue::from_bytes(t)?));
         #[cfg(not(any(feature = "use_hyper", feature = "use_async_h1")))]
-        return Err(imp::Error{})
+        return Err(imp::Error {});
     }
 }
 
@@ -142,7 +162,7 @@ impl<'a> From<&'a imp::HeaderValue> for &'a HeaderValue {
     #[inline]
     fn from(t: &'a imp::HeaderValue) -> Self {
         //safe because repr(transparent)
-        unsafe{std::mem::transmute(t)}
+        unsafe { std::mem::transmute(t) }
     }
 }
 impl AsRef<[u8]> for HeaderValue {
@@ -158,6 +178,7 @@ impl AsRef<[u8]> for HeaderValue {
 }
 impl std::convert::TryInto<String> for HeaderValue {
     type Error = std::string::FromUtf8Error;
+    #[cfg_attr(feature = "use_async_h1", allow(unreachable_code))]
     fn try_into(self) -> Result<String, Self::Error> {
         #[cfg(feature = "use_async_h1")]
         return Ok(self.0.as_str().to_string());
@@ -166,6 +187,7 @@ impl std::convert::TryInto<String> for HeaderValue {
 }
 impl std::convert::TryInto<String> for &HeaderValue {
     type Error = std::string::FromUtf8Error;
+    #[cfg_attr(feature = "use_async_h1", allow(unreachable_code))]
     fn try_into(self) -> Result<String, Self::Error> {
         #[cfg(feature = "use_async_h1")]
         return Ok(self.0.as_str().to_string());
@@ -173,6 +195,10 @@ impl std::convert::TryInto<String> for &HeaderValue {
     }
 }
 impl PartialEq<str> for HeaderValue {
+    #[cfg_attr(
+        not(any(feature = "use_hyper", feature = "use_async_h1")),
+        allow(unused_variables)
+    )]
     #[inline]
     fn eq(&self, other: &str) -> bool {
         #[cfg(feature = "use_hyper")]
@@ -185,6 +211,10 @@ impl PartialEq<str> for HeaderValue {
 }
 
 impl PartialEq<[u8]> for HeaderValue {
+    #[cfg_attr(
+        not(any(feature = "use_hyper", feature = "use_async_h1")),
+        allow(unused_variables)
+    )]
     #[inline]
     fn eq(&self, other: &[u8]) -> bool {
         #[cfg(feature = "use_hyper")]
